static uint32_t   previousTime;
static uint32_t   neutralizeTime;
static uint32_t   rcTime;
static uint32_t   currentTime;
static uint16_t   cycleTime;          // This is the number in micro second to achieve a full loop, it can differ a little and is taken into account in the PID loop
static uint16_t   meanTime = 2000;    // This is the average time of the loop: around 2ms for a WMP config and 6ms for a NK+WMP config
static uint16_t   calibratingA;       // To be more precise, the calibration is now done is the main loop. 
static uint16_t   calibratingG;       // Calibrating decreases at each cycle up to 0, then we enter in a normal mode.
                                      // we separate the calibration of ACC and gyro, because number of measure is not always equal.
static uint8_t    armed = 0;
static int16_t    acc_1G = 200;       //this is the 1G measured acceleration (nunchuk)
static int16_t    acc_25deg = 85;     //this is the the ACC value measured on x or y axis for a 25deg inclination (nunchuk) = acc_1G * sin(25)
static uint8_t    nunchukPresent = 0;
static uint8_t    accPresent = 0;     //I2C or ADC acc present
static uint8_t    gyroPresent = 0;    //I2C or ADC gyro present (other than WMP)
static uint8_t    magPresent = 0;     //I2C or ADC compass present
static uint8_t    baroPresent = 0;    //I2C or ADC baro present
static uint8_t    accMode = 0;        //if level mode is a activated
static uint8_t    magMode = 0;        //if compass heading hold is a activated
static uint8_t    baroMode = 0;       //if altitude hold is activated
static int16_t    accADC[3];
static int16_t    gyroADC[3];
static int16_t    magADC[3];
static int16_t    heading;
static int16_t    altitudeSmooth = 0;
static uint8_t    calibratedACC = 0;

// I2C Wii Motion Plus 
static uint8_t    rawADC_WMP[6];

// gyro+acc IMU
static int16_t gyroData[3] = {0,0,0};
static int16_t gyroZero[3] = {0,0,0};
static int16_t accZero[3]  = {0,0,0};
static int16_t angle[2];      //absolute angle inclination in multiple of 0.1 degree
static int16_t accSmooth[3];  //projection of smoothed and normalized gravitation force vector on x/y/z axis, as measured by accelerometer

// motor and servo functions
uint8_t PWM_PIN[6] = {MOTOR_ORDER};
  
static int16_t axisPID[3];
static int16_t motor[6];
static int16_t servo[4] = {1500,1500,1500,1500};
volatile uint8_t atomicServo[4] = {250,250,250,250};

//for HEX Y6 and HEX6 flat
volatile uint8_t atomicPWM_PIN5_lowState;
volatile uint8_t atomicPWM_PIN5_highState;
volatile uint8_t atomicPWM_PIN6_lowState;
volatile uint8_t atomicPWM_PIN6_highState;

// rc functions
#define MINCHECK 1100
#define MAXCHECK 1900

static uint8_t pinRcChannel[8] = {ROLLPIN, PITCHPIN, YAWPIN, THROTTLEPIN, AUX1PIN,AUX2PIN,CAM1PIN,CAM2PIN};
volatile uint16_t rcPinValue[8] = {1500,1500,1500,1500,1500,1500,1500,1500}; // interval [1000;2000]
static int16_t rcData[8] ; // interval [1000;2000]
static int16_t rcCommand[4] ; // interval [1000;2000] for THROTTLE and [-500;+500] for ROLL/PITCH/YAW 
static int16_t rcHysteresis[8] ;
static int16_t rcData4Values[8][4];

static uint8_t rcRate8;
static uint8_t rcExpo8;
static float rcFactor1; 
static float rcFactor2;

// ***PPM SUM SIGNAL***
#ifdef SERIAL_SUM_PPM
static uint8_t rcChannel[8] = {SERIAL_SUM_PPM};
#endif
volatile uint16_t rcValue[8] = {1500,1500,1500,1500,1500,1500,1500,1500}; // interval [1000;2000]

// EEPROM functions
static uint8_t P8[3], I8[3], D8[3]; //8 bits is much faster and the code is much shorter
static uint8_t dynP8[3], dynI8[3], dynD8[3]; 
static uint8_t PLEVEL8,ILEVEL8;
static uint8_t rollPitchRate;
static uint8_t yawRate;
static uint8_t dynThrPID;
static uint8_t checkNewConf = 133;
static uint8_t activateAcc8;
static uint8_t activateBaro8;
static uint8_t activateMag8;

